% Computes the persistence of a function f on a surface, and the
% association between every vertex of the surface and the k most prominent
% local maxima of f.
%
% Input: surface, f (real-valued function), k: integer
% Output: S -- a vector of indices where S(i) = j means that vertex i is
%              associated with the j^th most prominent local maximum.
%         P -- a vector of size k, with persistence values of the first k 
%              local maxima of f in increasing order.
%         I -- a vector of size k, with the indices of the vertices, whose
%              values correspond to k local maxima with the highest
%              persistence.
function [S, P, I] = pb_clustering(surface, f, k)

    [I, P] = mesh_persistence(surface, f);

    if(length(P)<k)
        error('Max number of clusters possible is: %d\n', length(P));
    end

    if (k == length(P))
        thresh = 0;
    else
        P = sort(P);

        thresh = (P(end-k) + P(end-k+1))/2;
    end
    
    edges = [surface.TRIV(:,[1 2]);
             surface.TRIV(:,[2 3]);
             surface.TRIV(:,[1 3])];
    edges = unique(sort(edges,2),'rows');
    
    [~, ~, S] = comp_graph_persistence(double(edges'), double(f), thresh);
   
    I2 = sparse(size(I));
    I2(I) = P;
    
    u = unique(S);
    P = I2(u)';
    I = u;
    
    code = 1:length(u);
    C = sparse(size(I));
    C(u) = code;
    
    S = C(S);    
end